#include "stdafx.h"



#include "openttd.h"
#include "currency.h"
#include "functions.h"
#include "spritecache.h"
#include "station.h"
#include "strings.h"
#include "table/sprites.h"
#include "table/strings.h"
#include "map.h"
#include "tile.h"
#include "window.h"
#include "gui.h"
#include "viewport.h"
#include "gfx.h"
#include "gfxinit.h"
#include "player.h"
#include "command.h"
#include "news.h"
#include "town.h"
#include "vehicle.h"
#include "console.h"
#include "sound.h"
#include "network.h"
#include "signs.h"
#include "waypoint.h"
#include "variables.h"
#include "train.h"
#include "unmovable_map.h"

#include "network_data.h"
#include "network_client.h"
#include "network_server.h"

#ifdef SBUILDTOWNS

#include "SBuildTowns.h"


#define		SBT_COST_TOWN			5000
#define		SBT_MAX_BUILDTOWNS		10

static int	sbt_newTown=0;
static Tile *sbt_map=NULL;


TileIndex SBT_GetSceen()
{
	Point pt=GetTileBelowCursor();
	TileIndex ind=TileVirtXY(pt.x, pt.y);
	return ind;
}
Town *SBT_CreateTown(char *NameTown,TileIndex tile, uint size_mode)
{
	StringID str;
	Town *city=SBT_CreateTownTile(tile,size_mode);
	if (!city)
	{
		//ShowErrorMessage(STR_EMPTY,STR_CANNOT_GENERATE_TOWN,50,50);
		return 0;
	}
	str = AllocateNameUnique(NameTown, 4);
	if (str == 0) return city;

	DeleteName(city->townnametype);
	city->townnametype = str;

	UpdateTownVirtCoord(city);
	_town_sort_dirty = true;
	UpdateAllStationVirtCoord();
	MarkWholeScreenDirty();
	return city;
}
static void SBT_PayBuildTown(TileIndex tile)
{
	Town *City=NULL;
		
	if (!_m)return;
	if (_m!=sbt_map){sbt_newTown=0;sbt_map=_m;}

	if (sbt_newTown>=SBT_MAX_BUILDTOWNS)
	{
		ShowErrorMessage(STR_EMPTY,STR_0302_TOWN_CONSTRUCTION,50,50);
		return;
	}

	if (!CheckPlayerHasMoney(SBT_COST_TOWN))
	{
		SetDParam(0, SBT_COST_TOWN);
		ShowErrorMessage(STR_0003_NOT_ENOUGH_CASH_REQUIRES,STR_EMPTY,50,50);
		return ;
	}

	_generating_world = true;
	City=SBT_CreateTown("Smokygrad",tile,1);
	_generating_world = false;
	if (City)
	{
		int32 pay=SBT_COST_TOWN;
		TileIndex ti=SBT_GetSceen();
		int x = TileX(ti) * TILE_SIZE;
		int y = TileY(ti) * TILE_SIZE;
		City->population=10;
		
		SubtractMoneyFromPlayer(pay);
		
		CreateEffectVehicleAbove(x +TILE_SIZE/2 , y+TILE_SIZE/2  , 0,EV_BULLDOZER);
		CreateEffectVehicleAbove(x -TILE_SIZE/2 , y+TILE_SIZE/2  , 0,EV_EXPLOSION_LARGE);
		CreateEffectVehicleAbove(x -TILE_SIZE/2 , y-TILE_SIZE/2  , 0,EV_EXPLOSION_LARGE);
		CreateEffectVehicleAbove(x +TILE_SIZE/2 , y-TILE_SIZE/2  , 0,EV_EXPLOSION_LARGE);
		
		CreateEffectVehicleAbove(x -TILE_SIZE/2 , y+TILE_SIZE/2+TILE_SIZE  , 0,EV_EXPLOSION_LARGE);
		CreateEffectVehicleAbove(x -TILE_SIZE/2+TILE_SIZE , y-TILE_SIZE/2+TILE_SIZE*2   , 0,EV_EXPLOSION_LARGE);
		CreateEffectVehicleAbove(x -TILE_SIZE/2+TILE_SIZE*2 , y-TILE_SIZE/2+TILE_SIZE*2   , 0,EV_EXPLOSION_LARGE);
		CreateEffectVehicleAbove(x -TILE_SIZE/2+TILE_SIZE*2 , y-TILE_SIZE/2+TILE_SIZE   , 0,EV_EXPLOSION_LARGE);
		CreateEffectVehicleAbove(x -TILE_SIZE/2+TILE_SIZE*2 , y-TILE_SIZE/2   , 0,EV_EXPLOSION_LARGE);
		if (pay != 0)
			ShowCostOrIncomeAnimation(x, y, GetSlopeZ(x, y), pay);
		sbt_newTown++;
		SndPlayTileFx(SND_12_EXPLOSION, tile);
	}
	
	return ;
}

static const Widget sbt_build_town_widgets[] = {
{   WWT_CLOSEBOX,   RESIZE_NONE,     7,     0,    10,     0,    13, STR_00C5,STR_018B_CLOSE_WINDOW},
{    WWT_CAPTION,   RESIZE_NONE,     7,    11,   31,     0,    13, STR_EMPTY	, STR_018C_WINDOW_TITLE_DRAG_THIS},
{  WWT_STICKYBOX,   RESIZE_NONE,     7,   32,   43,     0,    13, 0x0,     STR_STICKY_BUTTON},
{      WWT_PANEL,   RESIZE_NONE,     7,    0,     21,    14,    35, SPR_IMG_TOWN,		STR_0301_TOWN_CONSTRUCTION},
{      WWT_PANEL,   RESIZE_NONE,     7,    22,    43,    14,    35, SPR_IMG_BUY_LAND,        STR_0329_PURCHASE_LAND_FOR_FUTURE},
{   WIDGETS_END},
};


static const WindowDesc sbt_build_town_desc = {
	640-44, 22, 44, 36,
	WC_BUILD_TOOLBAR,0,
	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET | WDF_STICKY_BUTTON,
	sbt_build_town_widgets,
	SBT_BuildTownToolbWndProc
};
static void SBT_TerraformClick_BuyLand(Window *w)
{
	HandlePlacePushButton(w, 4, SPR_CURSOR_BUY_LAND, 1, PlaceProc_BuyLand);
}	

static void SBT_BuildTownClick(Window *w)
{
	HandlePlacePushButton(w, 3, SPR_CURSOR_TOWN, 1, SBT_PayBuildTown);
}

typedef void SBT_OnButtonClick(Window *w);
static SBT_OnButtonClick * const sbt_build_town_button_proc[] = 
{
	SBT_BuildTownClick,
	SBT_TerraformClick_BuyLand,
};

static void SBT_BuildTownToolbWndProc(Window *w, WindowEvent *e)
{
	switch (e->event) 
	{
	case WE_PAINT:
		w->disabled_state &= ~(1 << 16);
		if (!(w->click_state & ((1<<4)|(1<<5)|(1<<6)|(1<<7)|(1<<8)|(1<<11)|(1<<12)|(1<<13)|(1<<19)))) {
			w->disabled_state |= (1 << 16);
			w->click_state &= ~(1<<16);
		}
		DrawWindowWidgets(w);
		break;

	case WE_CLICK:
		if (e->click.widget >=3)
		{
			sbt_build_town_button_proc[e->click.widget-3](w);
		}
	break;

	case WE_KEYPRESS: 
	{
		break;
	}

	case WE_PLACE_OBJ:
	{
		_place_proc(e->place.tile);
	}
		return;

	case WE_PLACE_DRAG: 
	{
		VpSelectTilesWithMethod(e->place.pt.x, e->place.pt.y, e->place.userdata & 0xF);
		return;
	}

	case WE_PLACE_MOUSEUP:
//		_thd.height_follow = 0;
		if (e->click.pt.x != -1) 
		{
			TileIndex start_tile = e->place.starttile;
			TileIndex end_tile = e->place.tile;
		}
		break;

	case WE_ABORT_PLACE_OBJ:
		UnclickWindowButtons(w);
		SetWindowDirty(w);
		break;

	case WE_PLACE_PRESIZE: 
	{
		TileIndex tile = e->place.tile;
	} break;

	case WE_DESTROY:
		if (_patches.link_terraform_toolbar) DeleteWindowById(WC_SCEN_LAND_GEN, 0);
		break;
	}
}
void SBT_ShowBuildTownToolbar()
{
	Window *w;
	if (_current_player == OWNER_SPECTATOR) return;
	if (!(w = FindWindowById(WC_BUILD_TOOLBAR, 0)) || w->wndproc != SBT_BuildTownToolbWndProc) 
	{
		DeleteWindowById(WC_BUILD_TOOLBAR, 0);
		w = AllocateWindowDesc(&sbt_build_town_desc);
	}
	if (_patches.link_terraform_toolbar) ShowTerraformToolbar();
}



#endif SBUILDTOWNS